' *********************************************************************
' *                                                                   *
' * NOME SCRIPT: EnablePasswordChange.vbs                             *
' *                                                                   *
' * Script scritto da Alessandro Tani (Home Works S.p.A.)             *
' *                                                                   *
' * Versione 1.0 - Modificato l'ultima volta il 29/02/2008            *
' *                da Alessandro Tani                                 *
' * SCOPO:                                                            *
' *                                                                   *
' *  Impone a tutti gli utenti di una Unit Organizzativa la scadenza * 
' *  della password a tutti quegli utenti in cui la scadenza della    *
' *  password risultava disabilitata.                                 *
' *                                                                   *
' * File di log: strLogFile                                           *
' *                                                                   *
' * PREREQUISITI:                                                     *
' *                                                                   *
' *  L'utente che esegue lo script deve avere diritti amministrativi  *
' *  su Active Directory.                                             *
' *                                                                   *
' * TESTO:                                                            *
' *                                                                   *
' *  Nessuno                                                          *
' *                                                                   *
' *********************************************************************

'Dichiariamo sempre le variabili
Option Explicit

'Non gestiamo gli errori
'On Error Resume Next

'Impostiamo gli aggetti
Dim objArgs, objRootDSE, objFSO, objShell, objNetwork, objLogFile, objOU, objChildObject, objUser

'Impostiamo le variabili
Dim strDNOU, strDomain, strSystemDrive, strMese, strGiorno, strOra, strMinuti, strDataCorrente
Dim strLogFile, intUAC

'Impostiamo le costanti
Const ADS_ACETYPE_ACCESS_DENIED_OBJECT = &H6
Const ADS_ACETYPE_ACCESS_ALLOWED_OBJECT = &H5
Const ADS_FLAG_OBJECT_TYPE_PRESENT = &H1
Const ADS_RIGHT_DS_CONTROL_ACCESS = &H100
Const CHANGE_PASSWORD_GUID = "{ab721a53-1e2f-11d0-9819-00aa0040529b}"
const ADS_UF_DONT_EXPIRE_PASSWD = &H10000

'Leggiamo i parametri inseriti dall'utente
Set objArgs = WScript.Arguments

If objArgs.Count = 0 Then 'Messaggio di aiuto
	WScript.Echo ""
	WScript.Echo "Questo script abilita la scadenza delle password a tutti gli utenti,"
	WScript.Echo "di una Unit Organizzativa, per i quali la scadenza della password"
	WScript.Echo "era stata disabilitata."
	WScript.Echo ""
	WScript.Echo "Sintassi:"
	WScript.Echo ""
	WScript.Echo "   EnablePasswordChange <Distinguished_Name_OU>"
	WScript.Echo ""
	WScript.Echo "Dove:"
	WScript.Echo "  <Distinguished_Name_OU>:  il Distinguished Name che identifica la OU"
	WScript.Echo "                           in Active Directory"
	WScript.Echo ""
	WScript.Echo "Esempio:"
	WScript.Echo ""
	WScript.Echo "   EnablePasswordChange " & Chr(34) & "OU=Internal_Users,OU=My_Users,DC=homeworks,DC=it" & Chr(34)
	WScript.Echo ""
Else
	strDNOU = objArgs(0)  'Distinguished Name della OU in cui prelevare gli utenti

	'Definiamo gli oggetti
	Set objRootDSE = GetObject("LDAP://rootDSE")
	Set objFSO = WScript.CreateObject("Scripting.FileSystemObject")
	Set objShell = CreateObject("WScript.Shell")
	Set objNetwork = CreateObject("WScript.Network")
	strDomain = objNetwork.UserDomain

	'Creiamo la cartella in cui inserire il file di log
	strSystemDrive = objShell.ExpandEnvironmentStrings("%SystemDrive%")
	If not objFSO.FolderExists(strSystemDrive & "\Logs") then
		objFSO.CreateFolder(strSystemDrive & "\Logs")
	End If

	'Impostiamo la data e l'ora di esecuzione dello script
	If Len(DatePart("m", Date())) = 1 Then
		strMese = "0" & DatePart("m", Date())
	Else
		strMese = DatePart("m", Date())
	End If
	If Len(DatePart("d", Date())) = 1 Then
		strGiorno = "0" & DatePart("d", Date())
	Else
		strGiorno = DatePart("d", Date())
	End If
	If Len(Hour(Now())) = 1 Then
		strOra = "0" & Hour(Now())
	Else
		strOra = Hour(Now())
	End If
	If Len(Minute(Now())) = 1 Then
		strMinuti = "0" & Minute(Now())
	Else
		strMinuti = Minute(Now())
	End If

	'Impostiamo i file di log
	strDataCorrente = DatePart("yyyy", Date()) & strMese & strGiorno & strOra & strMinuti
	strLogFile = strSystemDrive & "\Logs\" & strDataCorrente & "_EnablePasswordChange.log"

	'Creiamo i file di log
	Set objLogFile = objFSO.CreateTextFile(strLogFile, True)

	'Impostiamo l'intestazione dei file di log
	objLogFile.WriteLine "+--------------------------------------------------+"
	objLogFile.WriteLine "|  File di log dello Script EnablePasswordChange   |"
	objLogFile.WriteLine "+--------------------------------------------------+"
	objLogFile.WriteLine ""
	objLogFile.WriteLine "Esecuzione del " & Date() & " alle " & Time()
	objLogFile.WriteLine ""

	'Eseguiamo la query in AD
	Wscript.Echo ""
	Wscript.Echo "Eseguiamo la query in AD per ottenere l'elenco degli utenti da controllare"
	Wscript.Echo "che si trovano nella OU: " & strDNOU
	Wscript.Echo ""
	objLogFile.WriteLine "Eseguiamo la query in AD per ottenere l'elenco degli utenti da controllare"
	objLogFile.WriteLine "che si trovano nella OU: " & strDNOU
	objLogFile.WriteLine ""

	'Otteniamo l'elenco degli oggetti della OU da analizzare
	Set objOU = GetObject("LDAP://" & strDNOU)
	For Each objChildObject in objOU
		Set objUser = GetObject(objChildObject.ADSPath)
		
		'Verifichiamo che l'oggetto sia un utente
		If objUser.Class = "user" Then
			Wscript.Echo " --> Controlliamo l'utente: " & objUser.Get("displayName")
			objLogFile.WriteLine " --> Controlliamo l'utente: " & objUser.Get("displayName")
			intUAC = objUser.Get("userAccountControl")

			If intUAC and ADS_UF_DONT_EXPIRE_PASSWD Then
				objUser.put "userAccountControl",  intUAC XOR ADS_UF_DONT_EXPIRE_PASSWD
				objUser.setinfo
				Wscript.Echo "     Impostiamo la scadenza della password all'utente"
				objLogFile.WriteLine "    Impostiamo la scadenza della password all'utente"
			Else
				Wscript.Echo "     La scadenza della password risulta gi abilitata"
				objLogFile.WriteLine "    La scadenza della password risulta gi abilitata"
			End If
			Wscript.Echo ""
			objLogFile.WriteLine ""
		End If
		
	Next

	'Chiudiamo i file aperti
	objLogFile.WriteLine "Fine del file di Log"
	objLogFile.Close

End If
